@@ -28,7 +28,7 @@ output = ARGV.join(" ") || output
 require 'socket'
 server = TCPServer.new("localhost", port)
 
-def p msg
+def p(msg)
   #puts "o:" + msg
   @s.print msg + "\r\n"
 end
