@@ -1,38 +1,39 @@
 module Schleuder
   class Mail < TMail::Mail
-    # The keywords (X-commands) collected from incoming mail
-    attr_accessor :keywords
+
     # Schleuder::Member's the mail shall be sent to
     attr_accessor :recipients
+
     # Additional data that is to be stored into internally sent
     # mails. Must be a Hash.
     attr_accessor :metadata
+
     # Indicator if the incoming mail was encrypted
     attr_accessor :in_encrypted
+
     # Indicator if (and by whom) the incoming mail was signed. Is +false+ or a
     # GPGME::Signature.
     attr_accessor :in_signed
+
     attr_accessor :resend_to
-    # An instance of Schleuder::Crypt
-    attr_accessor :crypt
+
     # The incoming  mail in raw form (string). Needed to verify detached
     # signatures (see +decrypt!+)
     attr_accessor :original_message
+
     # Shall we consider this email to be sent by the list-admin?
     attr_reader :from_admin
+
     # Shall we consider this email to be sent by a list-member?
     attr_reader :from_member
 
     # internal message_id we'll use for all outgoing mails
     @@schleuder_message_id = nil
 
-    # Instantiates some variables (a.o. @crypt)
     def initialize(*data)
       super(*data)
       @resend_to = []
-      @metadata = {}
-      @keywords = {}
-      @crypt = Crypt.new(Schleuder.list.config.gpg_password)
+      @metadata = {:resent_to => [], :notice => [], :warning => [], :error => []}
     end
 
     # Overwrites TMail.parse to store the original incoming data. We possibly
@@ -48,7 +49,7 @@ module Schleuder
     def decrypt!
       # Note: We don't recurse into nested Mime-parts. Only the first level
       # will be touched
-      if self.multipart? and !self.type_param('protocol').nil? and self.type_param('protocol').match(/^application\/pgp.*/)
+      if self.multipart? && !self.type_param('protocol').nil? && self.type_param('protocol').match(/^application\/pgp.*/)
         Schleuder.log.debug 'mail is pgp/mime-formatted'
         if self.sub_type == 'signed'
           Schleuder.log.debug 'mail is signed but not encrypted'
@@ -85,7 +86,6 @@ module Schleuder
           plainmsg.original_message = plaintext
           plainmsg.decrypt!
           Schleuder.log.debug "End of recursion"
-          #Schleuder.log.debug "plainmsg after decrypt: #{plainmsg.to_s.inspect}"
           self.in_signed = plainmsg.in_signed
         end
         # get headers and body(parts) into self
@@ -101,8 +101,6 @@ module Schleuder
         end
         self.content_type = plainmsg._content_type
         self.disposition = plainmsg._disposition
-        #Schleuder.log.debug "plainmsg: #{plainmsg.to_s}"
-        #Schleuder.log.debug "self after decrypt: #{self.to_s}"
       else
         Schleuder.log.debug 'no pgp-mime found, looking for pgp-inline'
         # Do we simply push everything to crypt as that should return plain
@@ -131,10 +129,13 @@ module Schleuder
 
       Schleuder.log.info "Encrypting for #{receiver.inspect}"
 
-      keystring = receiver.key || receiver.email
-      unless @crypt.get_key(keystring)
-        Schleuder.log.warn "No public key found for #{keystring}"
-        return false
+      key, msg = receiver.key
+      if key == false
+        Schleuder.log.warn msg.capitalize
+        return [false, msg]
+      elsif key.nil?
+        Schleuder.log.warn "No public key found for '#{keystring}'"
+        return [false, 'no public key found']
       end
 
       # choose pgp-variant from addresses value oder config.default_mime
@@ -169,8 +170,7 @@ module Schleuder
           gpgcontent.charset = self.charset || 'UTF-8'
         end
 
-        #Schleuder.log.debug "mail right before encrypting: #{gpgcontent.to_s}"
-        gpgcontainer = _new_part(@crypt.encrypt_str(gpgcontent.to_s, receiver), 'application/octet-stream', self.charset)
+        gpgcontainer = _new_part(crypt.encrypt_str(gpgcontent.to_s, receiver), 'application/octet-stream', self.charset)
         gpgcontainer.set_disposition('inline', {:filename => 'message.asc'})
 
         mimecontainer = _new_part("Version: 1\n", 'application/pgp-encrypted')
@@ -182,6 +182,7 @@ module Schleuder
         self.parts.push(mimecontainer)
         self.parts.push(gpgcontainer)
         self.set_content_type('multipart', 'encrypted', {:protocol => 'application/pgp-encrypted'})
+        self.mime_version = 1.0
 
       when 'APPL'
         Schleuder.log.error "mime-setting 'APPL' is deprecated, use 'MIME' instead for user '#{receiver.email}'"
@@ -195,12 +196,12 @@ module Schleuder
     # changed anymore) or a Schleuder::Mail.
     def sign
       Schleuder.log.debug "signing message"
-      if self.to.length > 1
-        Schleuder.log.error "More than one recipient in To-header"
+      if (self.to || []).length != 1
+        Schleuder.log.error "I need exactly one recipient in To-header. Found this: #{self.to.inspect}"
         return false
       end
       member = Schleuder.list.find_member_by_email(self.to.to_s)
-      # ugly but neccessary: member might be false and member.mime might be nil
+      # ugly but necessary: member might be false and member.mime might be nil
       pgpvariant = member.mime || Schleuder.list.config.default_mime rescue Schleuder.list.config.default_mime
       case pgpvariant.upcase
       when 'MIME'
@@ -208,16 +209,28 @@ module Schleuder
         gpgcontainer = Mail.new
         if self.multipart?
           self.parts.each do |p|
-            gpgcontainer.parts.push(Mail.parse(p.to_s))
+            if p.main_type.eql?('text')
+              # Encode QP (there might be trailing white space
+              p.body = p.body.to_s.split("\n").map do |line|
+                [line].pack("M")
+              end.join("\n")
+              p.encoding = 'Quoted-Printable'
+            end
+            gpgcontainer.parts.push(p)
           end
           self.parts.clear
         else
-          gpgcontainer.body = self.body.to_s
+          # Encode QP (there might be trailing white space which is illegal
+          # according to RFC 3156).
+          # Don't call body() as that returns the decoded string
+          gpgcontainer.body = body.to_s.split("\n").map do |line|
+              [line].pack("M")
+          end.join("\n")
+          gpgcontainer.encoding = 'Quoted-Printable'
           gpgcontainer.content_type = self._content_type
           gpgcontainer.disposition = self._disposition
           self.body = ''
         end
-
         self.encoding = ''
         self.parts.push gpgcontainer
 
@@ -230,7 +243,7 @@ module Schleuder
         # First we create the signature-attachment and fill it with a dummy
         dummy = "dummytobereplaced-#{Time.now.to_f}"
         sigpart = _new_part(dummy, 'application/pgp-signature')
-        self.parts.push sigpart
+        self.parts.push(sigpart)
 
         # TODO: take care of micalg: the digest used for hashing the plaintext.
         # RFC 3156 requires it to be set in the content-type. ruby-gpgme
@@ -242,33 +255,39 @@ module Schleuder
         rawmsg = self.encoded
 
         # get mime boundary from raw mail
-        bm = rawmsg.match(/boundary="?'?([^"';]*)/)
+        bm = rawmsg.match(/boundary="?'?([^"';\r\n]*)/)
         boundary = "--" + Regexp.escape(bm[1])
+
         # Now get the to be signed string from the raw message
-        tobesigned = rawmsg.match(/.*#{boundary}\r?\n(.*?)\r?\n#{boundary}.*?#{boundary}.*?/m)[1]
-        # For some reason I don't manage to do this in one gsub-call... *snif*
-        tobesigned_crlf = tobesigned.gsub(/\n/, "\r\n").gsub(/\r\r/, "\r")
-        # For another stupid reason tobesigned doesn't match -- although it was
-        # extracted from rawmsg just a few seconds ago...
-        rawmsg.gsub!(/(.*#{boundary}\r?\n).*?(\r?\n#{boundary}.*?#{boundary}.*?)/m, "\\1#{tobesigned_crlf}\\2").inspect
+        parts = rawmsg.match(/(.*#{boundary}\r?\n)(.*\r?\n)(\r?\n#{boundary}.*?#{boundary}.*)/m)
 
-        # sign the extracted part
-        sig = self.crypt.sign(tobesigned_crlf)
-        # replace dummy with signature
-        rawmsg.gsub!(/(.*)#{dummy}(.*?#{boundary}.*?)/m, "\\1#{sig}\\2")
-        rawmsg
+        if parts
+          # For some reason I don't manage to do this in one gsub-call... *snif*
+          tobesigned_crlf = parts[2].gsub(/\n/, "\r\n").gsub(/\r\r/, "\r")
+          rawmsg = "#{parts[1]}#{tobesigned_crlf}#{parts[3]}"
 
+          # sign the extracted part
+          sig = self.crypt.sign(tobesigned_crlf)
+
+          # replace dummy with signature
+          rawmsg.gsub!(/(.*)#{dummy}(.*?#{boundary}.*?)/m, "\\1#{sig}\\2")
+          rawmsg
+        else
+          Schleuder.log.error "Detection of mime parts in mail for #{self.to} with boundary #{bm} failed. This should not happen. -> Cannot sign outgoing mail."
+
+          false
+        end
       when 'PLAIN'
         Schleuder.log.debug "signing PLAIN"
         if self.multipart?
           self.parts.each_with_index do |p,i|
-            if p.disposition == 'attachment' or ! p.disposition_param('filename').nil?
+            if p.disposition == 'attachment' || !p.disposition_param('filename').nil?
               Schleuder.log.debug "found attachment, creating detached signature"
               # attachment, need detached sig
               container = Mail.new
               container.parts.push Mail.parse(p.to_s)
 
-              sig = self.crypt.sign(p.body)
+              sig = crypt.sign(p.body)
               Schleuder.log.debug "sig: #{sig.inspect}"
               sigpart = _new_part(sig, 'text/plain')
               sigpart.set_disposition('attachment', {:filename => p.disposition_param('filename') + '.asc'})
@@ -278,11 +297,11 @@ module Schleuder
 
               self.parts[i] = Mail.parse(container.to_s)
             else
-              p.body = self.crypt.clearsign(p.body)
+              p.body = crypt.clearsign(p.body)
             end
           end
         else
-          self.body = self.crypt.clearsign(self.body)
+          self.body = crypt.clearsign(self.body)
         end
         self
 
@@ -294,8 +313,9 @@ module Schleuder
 
     # Create a new Schleuder::Mail-instance and copy/set selected headers.
     def individualize(recv)
+      Schleuder.log.debug { "Individualizing mail for #{recv.inspect}" }
       new = Mail.new
-      new.crypt = @crypt
+      new.crypt = crypt
       # add some headers we want to keep
       new.content_type = self['content-type'].to_s if self['content-type']
       new.disposition = self._disposition
@@ -325,49 +345,71 @@ module Schleuder
     end
 
     # Strips keywords from self and stores them in @+keywords+
-    def collect_keywords!
-      # we're looking for keywords only in the first mime-part
-      b = self
-      while b.multipart?
-        b = b.parts.first
-      end
-      # split to array to ease parsing
-      a = b.body.to_a
+    def keywords
+      unless @keywords
+        @keywords = []
+        # we're looking for keywords only in the first mime-part
+        b = self
+        while b.multipart?
+          b = b.parts.first
+        end
+        # split to array to ease parsing
+        a = b.body.split("\n")
 
-      a.map! do |line|
-        if line.match(/^X-.*/)
-          Schleuder.log.debug "found keyword: #{line.chomp}"
-          key, val = $&.split(':')
-          if not self.from_admin and Schleuder.list.config.keywords_admin_only.include? key.slice(2..-1)
-            Schleuder.log.info "keyword is listed as admin only and mail is not from admin, skipping"
-            # return the line to have it stay in the body
+        a.map! do |line|
+          if line.match(/^X-.*/)
+            Schleuder.log.debug "found keyword-line: #{line.chomp}"
+            key, val = $&.split(/:|: | /, 2)
+            keyword = key.slice(2..-1).strip
+            if !self.from_admin && Schleuder.list.config.keywords_admin_only.include?(keyword)
+              Schleuder.log.info "Keyword '#{keyword}' is listed as admin only and mail is not from admin, skipping"
+              self.metadata[:error] << "Keyword '#{keyword}' is configured as admin-only."
+              # return the line to have it stay in the body
+              line
+            else
+              # Split values to catch multiple values separated by comma or the like (deprecated).
+              values = val.to_s.strip.split(/[ ,;]+/)
+              values << ' ' if values.empty?
+              values.each do |v|
+                Schleuder.log.info "Storing keyword #{keyword} with value #{v.inspect}"
+                @keywords << {keyword => v.to_s.strip}
+              end
+              nil
+            end
+          elsif line.chomp.empty?
             line
           else
-            Schleuder.log.info "Storing keyword #{key}"
-            val = '' if val.nil?
-            if @keywords[key]
-              @keywords[key] += ",#{val.strip}"
-            else
-              @keywords[key] = val.strip
-            end
-            nil
+            # break on the first non-empty and non-command line so we don't parse
+            # the whole message (could be much data)
+            break
           end
-        elsif line.chomp.empty?
-          line
-        else
-          # break on the first non-empty and non-command line so we don't parse
-          # the whole message (could be much data)
-          break
         end
+        # delete nil's (lines formerly containing X-Commands) from array
+        a.compact!
+        # rejoin to string
+        b.body = a.join("\n")
+        # The whole procedure makes TMail parse and reformat the message, so now
+        # it's decoded utf-8
+        b.charset = 'UTF-8'
+        b.encoding = nil
+        Schleuder.log.debug "Inspecting found keywords: #{@keywords.inspect}"
       end
-      # delete nil's (lines formerly containing X-Commands) from array
-      a.compact!
-      # rejoin to string
-      b.body = a.join("")
-      # The whole procedure makes TMail parse and reformat the message, so now
-      # it's decoded utf-8
-      b.charset = 'UTF-8'
-      b.encoding = nil
+      @keywords
+    end
+
+    def request?
+      @request ||= self.to.to_a.include?(Schleuder.list.request_addr)
+    end
+
+    def bounce?
+      self.to.to_a.include?(Schleuder.list.bounce_addr) || \
+        ( # empty Return-Path
+          ['<>'].include?(self['Return-path'].to_s) || \
+          ( # Auto-Submitted exists and does not equal 'no'
+            ! ["no", ""].include?(self['Auto-Submitted'].to_s) && \
+            self['X-Cron-Env'].nil? # cron mails are also autosubmitted
+          )
+        )
     end
 
     # +require+s all found plugins, tests SomePlugin.match and if that returns
@@ -376,19 +418,69 @@ module Schleuder
       if self.keywords.empty?
         Schleuder.log.info 'No keywords present, skipping plugins'
       elsif File.directory? Schleuder.config.plugins_dir
+        replymsg = ""
+        # We might want to replace the object later, which we can't do with self.
+        mail = self
+        plugins = {:request => [], :list => []}
+        ptype = (self.request? ? :request : :list)
+        used_keywords = []
+        Plugin.signing_key(self)
         Dir[Schleuder.config.plugins_dir + '/*_plugin.rb'].each do |plugfile|
-          Schleuder.log.debug "processing file #{plugfile} as plugin"
+          Schleuder.log.debug "Instanciating #{plugfile} as plugin"
           require plugfile
           # interpreting class name from file name
           classname = File.basename(plugfile, '.rb').split('_').collect { |p| p.capitalize }.join
           plugin = instance_eval(classname).new
-          Schleuder.log.debug "Testing #{plugin.class}.match..."
-          if plugin.match(self)
-            Schleuder.log.debug "#{plugin.class}.match matched -- executing #{plugin.class}.process"
-            plugin.process(self)
-          else
-            Schleuder.log.debug "#{plugin.class}.match didn't match"
+          Schleuder.log.debug "Storing plugin in plugin-list '#{plugin.plugin_type}'"
+          plugins[plugin.plugin_type] << plugin
+        end
+        # This is neither elegant nor fast, but it's got to live only until the rewrite.
+        Schleuder.log.debug "Processing #{ptype}-plugins"
+        self.keywords.each do |kwhash|
+          keyword = kwhash.keys.first
+          value = kwhash.values.first
+          Schleuder.log.debug "Looping for keyword #{keyword}"
+          plugins[ptype].each do |plugin|
+            command = keyword.downcase.gsub(/-/, '_')
+            Schleuder.log.debug "Does #{plugin.class}.respond_to? '#{command}'"
+            if plugin.respond_to?(command)
+              Schleuder.log.debug "Yes it does, executing #{plugin.class}.#{command}"
+              used_keywords << keyword
+              foo = plugin.send(command, mail, value)
+              case foo
+              when String
+                Schleuder.log.debug "Method returned a string, saving it for reply to sender."
+                replymsg << foo << "\n\n\n"
+              when Mail
+                Schleuder.log.debug "Method returned a Mail-object, replacing myself by it."
+                mail = foo
+              else
+                Schleuder.log.debug "Method returned '#{foo.inspect}', don't know how to handle, skipping."
+              end
+              next
+            else
+              Schleuder.log.debug "No it doesn't."
+            end
           end
+          # Generate error-message if no plugin was executed
+          unless used_keywords.include?(keyword)
+            msg = "No #{ptype}-plugin responded to keyword #{keyword}"
+            Schleuder.log.debug msg
+            if mail.request?
+              replymsg << "Error: #{msg}.\n\n"
+            else
+              mail.metadata[:error] << msg
+            end
+          end
+        end
+        # Decide how to go on: reply or list?
+        if mail.request?
+          Schleuder.log.debug "Message is a request, sending plugins-output back to sender."
+          if replymsg.empty?
+            replymsg = 'The keywords you sent did not produce any output. If you feel this is an error please contact the adminisrators of this list.'.fmt
+          end
+          # This exits.
+          Plugin.reply(mail, replymsg, used_keywords)
         end
       else
         Schleuder.log.error "#{Schleuder.config.plugins_dir} does not exist or is not readable!"
@@ -434,7 +526,7 @@ module Schleuder
       meta << "Sig: #{_sig_str @in_signed}\n"
       if self.multipart?
         self.parts.each_with_index do |p,i|
-          unless p.in_encrypted.nil? and p.in_signed.nil?
+          unless p.in_encrypted.nil? && p.in_signed.nil?
             meta << "part #{i+1}:\n"
             meta << "  enc: #{_enc_str p.in_encrypted}\n"
             meta << "  sig: #{_sig_str p.in_signed}\n"
@@ -442,13 +534,10 @@ module Schleuder
         end
       end
 
-      unless @metadata.empty?
-        Schleuder.log.debug 'Adding extra meta data'
-        @metadata.each do |name, content|
-          meta << "#{name.gsub(/_/, '-').capitalize}: #{content.to_s}\n"
-        end
+      Schleuder.log.debug 'Adding extra meta data'
+      @metadata.each do |name, content|
+        meta << "#{name.to_s.gsub(/_/, '-').capitalize}: #{(content.join('; ') || content.to_s)}\n" unless content.empty?
       end
-
       meta << "\n"
 
       # insert oder prepend to the message
@@ -471,8 +560,11 @@ module Schleuder
     # or the body of the first mimepart (if one of those is text/plain) or
     # appends it as a new mimepart.
     def add_public_footer!
+      if Schleuder.list.config.public_footer.strip.empty?
+        return false
+      end
       Schleuder.log.debug "appending public footer"
-      footer = "\n\n-- \n" + Schleuder.list.config.public_footer
+      footer = "\n\n-- \n#{Schleuder.list.config.public_footer}\n"
 
       if self.first_part._content_type_stripped == 'text/plain'
         self.first_part.body = self.first_part.body.to_s + footer
@@ -494,7 +586,6 @@ module Schleuder
       self.body = ''
       self.set_content_type 'multipart', 'mixed', {:boundary => TMail.new_boundary}
       self.disposition = ''
-      #Schleuder.log.debug self.to_s
     end
 
     def first_part
@@ -510,17 +601,33 @@ module Schleuder
     end
 
     def _content_type_stripped(default = nil)
-      ( default and _content_type(default) or _content_type() ).to_s.split(';').first
+      ( default && _content_type(default) || _content_type() ).to_s.split(';').first
     end
 
     def _disposition
       (self['content-disposition'] || '').to_s rescue ''
     end
 
+   def from_member_address?
+      from.all? { |a| Schleuder.list.find_member_by_address(a) }
+    end
+
+    def from_admin_address?
+      from.all? { |a| Schleuder.list.find_admin_by_address(a) }
+    end
+
+    # An instance of Schleuder::Crypt
+    attr_writer :crypt
+    def crypt
+      @crypt ||= Crypt.new(Schleuder.list.config.gpg_password)
+    end
+ 
     private
 
     def _decrypt_pgp_inline(msg)
-      if msg.body =~ /^-----BEGIN PGP.*/
+      if msg._content_type_stripped == 'text/html'
+        Schleuder.log.debug "Content-type is text/html, can't handle that, skipping"
+      elsif msg.body =~ /^-----BEGIN PGP.*/ || msg.body.content_type.split('/').last.eql?('pgp')
         Schleuder.log.debug 'found pgp-inline in input'
         # Look for charset-armor-header. In most cases useless but nobody shall say we didn't try.
         msg.body.each_line do |l|
@@ -528,20 +635,46 @@ module Schleuder
           next unless m = l.match(/^Charset:\s(.*)$/)
           @charset = m[1]
         end
-        msg.body, msg.in_encrypted, msg.in_signed = @crypt.decrypt(msg.body)
-        if msg.content_type and msg['content-type'].params['x-action'] =~ /^pgp/
+        # TMail decodes QP if body() is called, Umlauts if to_s() is called.
+        # Life with TMail is hard...
+        if msg.encoding.to_s.downcase.eql?('quoted-printable') || !msg.main_type.eql?('text')
+          str = msg.body
+        else
+          str = msg.to_s
+        end
+        # We need to do this in three steps:
+        # 1. get the decoded body from TMail
+        # 2. delete the encoding-header
+        # 3. put the plaintext back into TMail
+        # Else TMail either can't decode the body correctly or 'over-decodes' it
+        # on next output
+        ptxt, enc, sig = crypt.decrypt(str)
+        if msg.main_type.eql?('text')
+          msg.body = ptxt
+          msg.charset = @charset || 'UTF-8'
+          msg.encoding = nil
+        else
+          # base64-encode if not text as we don't know what's in there. Might
+          # be binary data, which could break otherweise.
+          msg.body = [ptxt].pack("m")
+          msg.encoding = 'base64'
+        end
+        msg.in_encrypted = enc
+        msg.in_signed = sig
+        # RFC 2440 defines that the armored text by default is utf-8
+        if msg.content_type && msg['content-type'].params['x-action'] =~ /^pgp/
           msg['content-type'].params['x-action'] = nil
         end
-        # RFC 2440 defines that the armored text by default is utf-8
-        msg.charset = @charset || 'UTF-8'
+        unless msg.disposition_param('filename').nil?
+          msg['content-disposition'].params['filename'] =  msg.disposition_param('filename').gsub(/\.(gpg|pgp|asc)$/, '')
+        end
       else
         Schleuder.log.debug 'no pgp-inline-data found, doing nothing'
       end
     end
 
     def _encrypt_pgp_inline(msg, receiver)
-      #Schleuder.log.debug "encoding: #{msg.encoding}"
-      msg.body = @crypt.encrypt_str(msg.body.to_s, receiver)
+      msg.body = crypt.encrypt_str(msg.body.to_s, receiver)
       # reset encoding, TMail has converted the content
       msg.encoding = nil
       # if attachment add '.gpg'-suffix to attachments file names
@@ -558,13 +691,13 @@ module Schleuder
       msg
     end
 
-    def _enc_str arg
-      arg and 'encrypted' or 'unencrypted'
+    def _enc_str(arg)
+      arg ? 'encrypted' : 'unencrypted'
     end
 
-    def _sig_str arg
+    def _sig_str(arg)
       if arg
-        if self.crypt.get_key(arg.fpr)
+        if crypt.get_key(arg.fpr).first
           arg.to_s
         else
           #Schleuder.log.debug arg.inspect
@@ -576,31 +709,29 @@ module Schleuder
     end
 
     # Tests if the signature of the incoming mail (if any) belongs to a
-    # list-member or adminaddr by testing all gpg-key-ids of the signing
+    # list-member or an admin by testing all gpg-key-ids of the signing
     # key for matches
     def _parse_signature
-      @from_admin = false
-      @from_member = false
+      @from_admin = @from_member = false
 
       Schleuder.log.debug 'Testing for valid signature'
-      if ! @in_signed
+      if !in_signed
         Schleuder.log.info 'No signature found'
-      elsif @in_signed.status != 0
+      elsif in_signed.status != 0
         Schleuder.log.info 'Invalid or unknown signature found'
       else
         Schleuder.log.info 'Valid signature found'
-        if key = @crypt.get_key(@in_signed.fpr)
-          Schleuder.log.debug 'Testing UIDs from key for membership or matching adminaddr'
-          key.uids.each do |u|
-            if Schleuder.list.is_admin?(u.email) or Schleuder.list.is_admin?(u.name)
-              Schleuder.log.debug "#{u.inspect} matched as adminaddr"
-              @from_admin = true
-            end
-            if Schleuder.list.is_member?(u.email) or Schleuder.list.is_member?(u.name)
-              Schleuder.log.debug "#{u.inspect} matches as member"
-              @from_member = true
-            end
+        key, msg = crypt.get_key(in_signed.fpr)
+        if key
+          Schleuder.log.debug "Testing key for matching some member's or admin's keys"
+          if Schleuder.list.find_admin_by_key(key)
+            @from_admin = true
           end
+          if Schleuder.list.find_member_by_key(key)
+            @from_member = true
+          end
+        else
+          Schleuder.log.debug "Keylookup for signature failed! Reason: #{msg}"
         end
       end
     end
@@ -647,12 +778,12 @@ module Schleuder
     end
 
     def _collect_message_ids(ids)
-      return nil if ids.nil? or not ids.is_a?(Array) or ids.empty?
-      ids.select{ |id| Utils.schleuder_id?(id,Schleuder.list.listname) }
+      return nil if ids.nil? || !ids.is_a?(Array) || ids.empty?
+      ids.select{ |id| Utils.schleuder_id?(id, Schleuder.list.listid) }
     end
 
     def self._schleuder_message_id
-      @@schleuder_message_id = Utils.generate_message_id(Schleuder.list.listname) unless @@schleuder_message_id
+      @@schleuder_message_id = Utils.generate_message_id(Schleuder.list.listid) unless @@schleuder_message_id
       @@schleuder_message_id
     end
 
@@ -665,17 +796,31 @@ module Schleuder
 
     def _list_headers(mail)
       Schleuder.log.debug "Generating list-ids"
-      mail['List-Id'] = _list_id
+      mail['List-Id'] = "<#{Schleuder.list.listid}>"
+      mail['List-Owner'] = _list_owner
+      mail['List-Post'] = _list_post
+      # TODO: adapt URL to version.
+      mail['List-Help'] = '<https://schleuder2.nadir.org/documentation.html>'
       mail
     end
 
-    def _list_id
-      "<#{Schleuder.list.config.myaddr.gsub(/@/,'.')}>"
+    def _list_owner
+      "<mailto:#{Schleuder.list.owner_addr}> (Use list's public key)"
+    end
+
+    def _list_post
+      if Schleuder.list.config.receive_admin_only
+        "NO (Admins only)"
+      elsif Schleuder.list.config.receive_authenticated_only
+        "<mailto:#{Schleuder.list.config.myaddr}> (Subscribers only)"
+      else
+        "<mailto:#{Schleuder.list.config.myaddr}>"
+      end
     end
 
     def _add_openpgp_header(mail)
       Schleuder.log.debug "Add OpenPGP-Headers"
-      mail['OpenPGP'] = "id=#{@crypt.get_key(Schleuder.list.config.myaddr).subkeys.first.keyid} "+
+      mail['OpenPGP'] = "id=#{Schleuder.list.key_fingerprint} "+
                         "(Send an email to #{Schleuder.list.sendkey_addr} to receive the public-key)"+
                         _gen_openpgp_pref_header
       mail
@@ -684,7 +829,9 @@ module Schleuder
     def _gen_openpgp_pref_header
       unless Schleuder.list.config.openpgp_header_preference == 'none'
         pref_str = "; preference=#{Schleuder.list.config.openpgp_header_preference} ("
-        if !Schleuder.list.config.receive_authenticated_only
+        if Schleuder.list.config.receive_admin_only
+          pref_str << 'Only encrypted and signed emails by list-admins are accepted'
+        elsif !Schleuder.list.config.receive_authenticated_only
           if Schleuder.list.config.receive_encrypted_only \
               && Schleuder.list.config.receive_signed_only
             pref_str << 'Only encrypted and signed emails are accepted'
