@@ -25,19 +25,23 @@ module Schleuder
     end
 
     def self.get_pretty_fingerprint(key)
-      "No such key!" if key.nil?
+      "Wrong input, need GPGME::Key!" unless key.kind_of?(GPGME::Key)
       key.subkeys.first.fingerprint.gsub(/(.{4})/, "\\1 ").strip
     end
 
-    def self.generate_message_id(listname='')
-      listname = listname + '-' unless listname.empty?
-      '<'+Utils.generate_random_str+"@#{listname}schleuder>"
+    def self.generate_message_id(listid='')
+      listid = listid + '-' unless listid.empty?
+      '<'+Utils.generate_random_str+"@#{listid}schleuder>"
     end
 
-    def self.schleuder_id?(message_id,listname='')
+    def self.schleuder_id?(message_id,listid='')
       return false unless message_id
-      listname = listname + '-' unless listname.empty?
-      /<\w*@#{Regexp.quote(listname)}schleuder>/ === message_id
+      listid = listid + '-' unless listid.empty?
+      /<\w*@#{Regexp.quote(listid)}schleuder>/ === message_id
+    end
+
+    def self.compress_fingerprint(fpr)
+      fpr =~ / / ? fpr.gsub(/ /,'') : fpr
     end
   end
 end
@@ -64,3 +68,13 @@ class Hash
     end
   end
 end
+
+class String
+  def fmt(linewidth=nil)
+    if linewidth.nil?
+      require 'highline/system_extensions'
+      linewidth = HighLine::SystemExtensions.terminal_size.first || 76
+    end
+    gsub(/(.{1,#{linewidth}})(\s+|$)/, "\\1\n")
+  end
+end
